"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var core = _interopRequireWildcard(require("../../core"));

var _BlurXFilter = _interopRequireDefault(require("./BlurXFilter"));

var _BlurYFilter = _interopRequireDefault(require("./BlurYFilter"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * The BlurFilter applies a Gaussian blur to an object.
 * The strength of the blur can be set for x- and y-axis separately.
 *
 * @class
 * @extends PISI.Filter
 * @memberof PISI.filters
 */
class BlurFilter extends core.Filter {
  /**
   * @param {number} strength - The strength of the blur filter.
   * @param {number} quality - The quality of the blur filter.
   * @param {number} resolution - The resolution of the blur filter.
   * @param {number} [kernelSize=5] - The kernelSize of the blur filter.Options: 5, 7, 9, 11, 13, 15.
   */
  constructor(strength, quality, resolution, kernelSize) {
    super();
    this.blurXFilter = new _BlurXFilter.default(strength, quality, resolution, kernelSize);
    this.blurYFilter = new _BlurYFilter.default(strength, quality, resolution, kernelSize);
    this.padding = 0;
    this.resolution = resolution || core.settings.RESOLUTION;
    this.quality = quality || 4;
    this.blur = strength || 8;
  }
  /**
   * Applies the filter.
   *
   * @param {PISI.FilterManager} filterManager - The manager.
   * @param {PISI.RenderTarget} input - The input target.
   * @param {PISI.RenderTarget} output - The output target.
   */


  apply(filterManager, input, output) {
    var renderTarget = filterManager.getRenderTarget(true);
    this.blurXFilter.apply(filterManager, input, renderTarget, true);
    this.blurYFilter.apply(filterManager, renderTarget, output, false);
    filterManager.returnRenderTarget(renderTarget);
  }
  /**
   * Sets the strength of both the blurX and blurY properties simultaneously
   *
   * @member {number}
   * @default 2
   */


  get blur() {
    return this.blurXFilter.blur;
  }

  set blur(value) {
    this.blurXFilter.blur = this.blurYFilter.blur = value;
    this.padding = Math.max(Math.abs(this.blurXFilter.strength), Math.abs(this.blurYFilter.strength)) * 2;
  }
  /**
   * Sets the number of passes for blur. More passes means higher quaility bluring.
   *
   * @member {number}
   * @default 1
   */


  get quality() {
    return this.blurXFilter.quality;
  }

  set quality(value) {
    this.blurXFilter.quality = this.blurYFilter.quality = value;
  }
  /**
   * Sets the strength of the blurX property
   *
   * @member {number}
   * @default 2
   */


  get blurX() {
    return this.blurXFilter.blur;
  }

  set blurX(value) {
    this.blurXFilter.blur = value;
    this.padding = Math.max(Math.abs(this.blurXFilter.strength), Math.abs(this.blurYFilter.strength)) * 2;
  }
  /**
   * Sets the strength of the blurY property
   *
   * @member {number}
   * @default 2
   */


  get blurY() {
    return this.blurYFilter.blur;
  }

  set blurY(value) {
    this.blurYFilter.blur = value;
    this.padding = Math.max(Math.abs(this.blurXFilter.strength), Math.abs(this.blurYFilter.strength)) * 2;
  }

  get blendMode() {
    return this.blurYFilter._blendMode;
  }

  set blendMode(value) {
    this.blurYFilter._blendMode = value;
  }

}

exports.default = BlurFilter;
//# sourceMappingURL=BlurFilter.js.map